package gov.usgs.earthquake.nshmp.netcdf.www;

import static com.google.common.base.CaseFormat.UPPER_CAMEL;
import static com.google.common.base.CaseFormat.UPPER_UNDERSCORE;

import java.lang.reflect.Type;
import java.util.logging.Logger;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonSerializationContext;
import com.google.gson.JsonSerializer;

import gov.usgs.earthquake.nshmp.Maths;
import gov.usgs.earthquake.nshmp.geo.Location;
import gov.usgs.earthquake.nshmp.gmm.Imt;
import gov.usgs.earthquake.nshmp.gmm.NehrpSiteClass;
import gov.usgs.earthquake.nshmp.www.Response;
import gov.usgs.earthquake.nshmp.www.WsUtils.EnumSerializer;
import gov.usgs.earthquake.nshmp.www.WsUtils.NaNSerializer;
import gov.usgs.earthquake.nshmp.www.meta.Status;

import io.micronaut.http.HttpResponse;

public class NetcdfWsUtils {
  static final Gson GSON;

  private static final Logger LOGGER = Logger.getLogger(NetcdfWsUtils.class.getName());

  static {
    GSON = new GsonBuilder()
        .registerTypeAdapter(Imt.class, new EnumSerializer<Imt>())
        .registerTypeAdapter(NehrpSiteClass.class, new EnumSerializer<NehrpSiteClass>())
        .registerTypeAdapter(Double.class, new NaNSerializer())
        .registerTypeAdapter(Location.class, new LocationSerializer())
        .disableHtmlEscaping()
        .serializeNulls()
        .setPrettyPrinting()
        .create();
  }

  public static HttpResponse<String> handleError(
      Throwable e,
      String name,
      String url) {
    var msg = e.getMessage() + " (see logs)";
    var svcResponse = new Response<>(Status.ERROR, name, url, msg, url);
    var response = GSON.toJson(svcResponse);
    LOGGER.severe(name + " -\n" + response);
    e.printStackTrace();
    return HttpResponse.serverError(response);
  }

  static enum Key {
    LATITUDE,
    LONGITUDE,
    SITE_CLASS,
    IMT;

    @Override
    public String toString() {
      return UPPER_UNDERSCORE.to(UPPER_CAMEL, name());
    }
  }

  public static final class LocationSerializer implements JsonSerializer<Location> {

    @Override
    public JsonElement serialize(Location location, Type type, JsonSerializationContext context) {
      JsonObject jObj = new JsonObject();
      jObj.addProperty("longitude", Maths.round(location.longitude, 5));
      jObj.addProperty("latitude", Maths.round(location.latitude, 5));

      return jObj;
    }
  }
}
